# encoding: utf-8

#*    dsgui - a graphical user interface for Datove schranky
#*    Copyright (C) 2009-2010  CZ.NIC, z.s.p.o. (http://www.nic.cz)
#*
#*    This library is free software; you can redistribute it and/or
#*    modify it under the terms of the GNU Library General Public
#*    License as published by the Free Software Foundation; either
#*    version 2 of the License, or (at your option) any later version.
#*
#*    This library is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#*    Library General Public License for more details.
#*
#*    You should have received a copy of the GNU Library General Public
#*    License along with this library; if not, write to the Free
#*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
#*    02110-1301  USA

"""this module is used for custom handling of exceptions that make it
to the top level exception handler"""

import traceback
import sys
import gtk

class ErrorDisplayDialog(gtk.Dialog):

  def __init__(self, message):
    gtk.Dialog.__init__(self, parent=None, title=_("Error message"),
                        buttons=(gtk.STOCK_OK, 1))
    mainbox = gtk.VBox()
    mainbox.set_border_width(10)
    self.vbox.pack_start(mainbox)
    # label 1
    l1 = gtk.Label(_("""<b>An exception occurred in dsgui</b>.
  
You might be interested in letting us know by sending the following debugging \
information along with description of when the error occurred to \
'<b><a href='mailto:datove-schranky@labs.nic.cz'>datove-schranky@labs.nic.cz\
</a></b>'.
This would help us fix this problem in future versions of dsgui.
<b>We apologize for any inconvenience caused by this problem.</b>"""))
    l1.set_use_markup(True)
    l1.set_selectable(True)
    l1.set_line_wrap(True)
    l1.set_alignment(0,0)
    l1.set_width_chars(80)
    mainbox.pack_start(l1, padding=10, fill=False, expand=False)
    l1.show()
    # debug text
    f1 = gtk.Frame(_("Debugging info"))
    a1 = gtk.Alignment(xscale=1, yscale=1)
    s1 = gtk.ScrolledWindow() 
    s1.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC) 
    self.textview = gtk.TextView()
    textbuffer = self.textview.get_buffer()
    textbuffer.set_text(message)
    s1.add(self.textview)
    #w, h = self.textview.size_request()
    a1.set_padding(10,10,10,10)
    a1.add(s1)
    f1.add(a1)
    mainbox.pack_start(f1)
    self.vbox.show_all()
    sh = self.get_screen().get_height()
    self.set_size_request(-1, min(sh-50, 600))


def show_exception_dialog(type, value, tb):
  tb_str = "".join( traceback.format_tb(tb))
  if hasattr(type, "__name__"):
    type_name = type.__name__
  else:
    type_name = str(type)
  message = "Type: %s\nValue: %s\nTraceback:\n%s" % (type_name, value, tb_str)
  print >> sys.stderr, message
  try:
    # add additional info
    message += "\n --- Additional info --- \n"
    message += "Python: %s\n" % ".".join([str(x) for x in sys.version_info])
    message += "Platform: %s\n" % sys.platform
    try:
      import release
      message += "dsgui version: %s\n" % release.DSGUI_VERSION
    except:
      message += "dsgui version: not available\n"
    try:
      import dslib.release
      message += "dslib version: %s\n" % dslib.release.DSLIB_VERSION
    except:
      message += "dslib version: not available\n"
    d = ErrorDisplayDialog(message)
    d.run()
    d.destroy()
  except Exception, e:
    print >> sys.stderr,\
       "An error has occurred when trying to display an error message :("
    print >> sys.stderr, e
  
if __name__ == "__main__":
  sys.excepthook = show_exception_dialog
  __builtins__["_"] = lambda x: x
  print AAA
  